(async function main() {
    const path = require("path");
    const fs = require("fs");
    const winston = require("winston");

    // Activate colorization for output
    winston.remove(winston.transports.Console);
    winston.add(winston.transports.Console, {
        colorize: true,
        level: "info",
        json: false,
        stringify: true,
    });

    // Setup variables to find the Makefile location
    const command = process.argv[2] || "all";
    const args = process.argv.slice(3);
    let fileName = "Makefile.js";
    const cwd = process.cwd();
    const packageJsonPath = path.resolve(cwd, "package.json");

    if (fs.existsSync(packageJsonPath)) {
        try {
            const packageJson = require(packageJsonPath);

            if (typeof packageJson.jmake === "string") {
                fileName = packageJson.jmake;
            }
        } catch (e) {
            winston.warn(`Unable to require '${packageJsonPath}'.`);
            winston.warn(`Defaulting fileName to ${fileName}.`);
            winston.warn(`Error: ${e.message}.`);
        }
    }

    const makefilePath = path.resolve(cwd, fileName);

    // Do the processing
    try {
        const makefile = require(makefilePath);
        if (command in makefile) {
            const handler = makefile[command];

            if (typeof handler !== "function") {
                winston.error("The command must be a function.");
                process.exit(4);
            }

            const result = handler(...args);

            Promise.resolve(result).catch(e => {
                winston.error("Something went wrong while executing the command.");
                winston.error(e.message);
                process.exit(3);
            });
        } else if (command === "-h" || command === "--help") {
            winston.info("Available commands:");
            Object.keys(makefile)
                .filter(cmd => typeof makefile[cmd] === "function")
                .map(cmd => `\t- ${cmd}`)
                .forEach(cmd => {
                    winston.info(cmd);
                });
            process.exit();
        } else {
            winston.error(`The command '${command}' does not exist in your Makefile.`);
            process.exit(2);
        }
    } catch (e) {
        winston.error(`Unable to require '${makefilePath}'.`);
        winston.error(`Error: ${e.message}.`);
        winston.error("Can't do anything. Exiting...");
        process.exit(1);
    }
})();
